﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class ConcatSynthGarbageImitate : MonoBehaviour 
{
	public Transform[] prefabs;

	public string[] soundFiles;

	public bool[] ignore;

	public Transform garbagePatches;

	private ChuckSubInstance myChuck;
	private ChuckIntSyncer myMatchedBuffer;
	private ChuckFloatSyncer myLoudness;
	private int previousObject = -1;

	private Transform[] myObjects;
	private Transform[] myPrefabs;
	private GarbagePatch[] myPatches;
	private float prevSpawnTime = 0;
	private bool shouldSpawnObjects = false;

	// Use this for initialization
	void Start()
	{
		InitChuck();
		InitObjects();

		myPatches = garbagePatches.GetComponentsInChildren<GarbagePatch>();
	}

	void InitObjects()
	{
		int numToIgnore = 0;
		for( int i = 0; i < ignore.Length; i++ )
		{
			if( ignore[i] ) { numToIgnore++; }
		}

		myPrefabs = new Transform[ prefabs.Length - numToIgnore ];
		myObjects = new Transform[ myPrefabs.Length ];
		int which = 0;
		for( int i = 0; i < prefabs.Length; i++ )
		{
			if( !ignore[i] )
			{
				myPrefabs[which] = prefabs[i];
				myObjects[which] = Instantiate( myPrefabs[which], transform );
				myObjects[which].localPosition = Vector3.zero;
				myObjects[which].gameObject.SetActive( false );
				which++;
			}
		}
	}

	void InitChuck()
	{
		myChuck = GetComponent<ChuckSubInstance>();

		string theFiles = "";
		for( int i = 0; i < soundFiles.Length; i++ )
		{
			if( !ignore[i] )
			{
				if( theFiles != "" )
				{
					theFiles += ", ";
				}
				theFiles += string.Format( @" ""{0}"" ", soundFiles[i] );
			}
		}

		myChuck.chuckMainInstance.RunCode( string.Format( @"
			// A BRIEF ASIDE FOR A CLASS~
			//-----------------------------------------------------------------------------
			// name: ChucKSupersawExperiments.ck
			// desc: experiments with making a supersaw class
			//
			// author: Jack Atherton (it me!)
			//-----------------------------------------------------------------------------
			class Supersaw extends Chubgraph
			{{
				SawOsc osc => Gain out =>  Gain ampMod => outlet;
				5 => int numDelays;
				1.0 / (1 + numDelays) => out.gain;
				DelayA theDelays[numDelays];
				SinOsc lfos[numDelays];
				dur baseDelays[numDelays];
				float baseFreqs[numDelays];
				for( int i; i < numDelays; i++ )
				{{
					osc => theDelays[i] => out;
					0.15::second => theDelays[i].max;
					// crucial to modify!
					Math.random2f( 0.001, 0.002 )::second => baseDelays[i];
					lfos[i] => blackhole;
					//        Math.random2f( -0.1, 0.1) => baseFreqs[i];
					Math.random2f( 0, pi ) => lfos[i].phase;
					
				}}
				0.05::second => dur baseDelay;
				0.333 => float baseFreq;
				1 => float lfoGain;
				
				fun void AttachLFOs()
				{{
					while( true )
					{{
						for( int i; i < numDelays; i++ )
						{{
							baseFreq + baseFreqs[i] => lfos[i].freq;
							lfoGain * lfos[i].last()::second + baseDelay + baseDelays[i] 
							=> theDelays[i].delay;
						}}
						1::ms => now;
					}}
				}}
				spork ~ AttachLFOs();
				
				
				SinOsc pitchLFO => blackhole;
				0.77 => pitchLFO.freq;
				1.0 / 300 => float pitchLFODepth;
				440 => float basePitch;
				0.13 => float ampModRate;
				
				fun void FreqMod()
				{{
					while( true )
					{{
						// calc freq
						basePitch + ( basePitch * pitchLFODepth ) 
						* pitchLFO.last() => float f;
						// set
						f => osc.freq;
						1.0 / f => lfoGain; // seconds per cycle == gain amount
						// wait
						1::ms => now;
					}}
				}}
				spork ~ FreqMod();
				
				fun void AmpMod()
				{{
					SinOsc lfo => blackhole;
					while( true )
					{{
						ampModRate => lfo.freq;
						0.85 + 0.15 * lfo.last() => ampMod.gain;
						1::ms => now;
					}}
				}}
				spork ~ AmpMod();

				
				fun void freq( float f )
				{{
					f => basePitch;
				}}
				
				fun void delay( dur d )
				{{
					d => baseDelay;
				}}
				
				fun void timbreLFO( float f )
				{{
					f => baseFreq;
				}}
				
				fun void pitchLFORate( float f )
				{{
					f => pitchLFO.freq;
				}}
				
				fun void pitchLFODepthMultiplier( float r )
				{{
					r => pitchLFODepth;
				}}
				
				fun void ampModLFO( float r )
				{{
					r => ampModRate;
				}}
			}}

			// NOW BACK TO OUR REGULARLY SCHEDULE CONCATENATION
			// Let's concatenative-synthesis!
			ConcatSynth concat => NRev reverbL => dac.left;
			concat => NRev reverbR => dac.right;
			0 => reverbL.mix => reverbR.mix;

			// 1. Optionally set number of MFCCs to be used in all analysis.
			//    13 is the default. 48 is the maximum. 
			//    We ignore the 0th MFCC, so you are actually using 
			//    one fewer than this.
			14 => concat.mfccCount;

			// 2. Add source sound files.
			[{0}] @=> string sources[];
			for( int i; i < sources.size(); i++ )
			{{
				concat.addSource( me.dir() + ""objectSounds/"" + sources[i] );
			}}
			concat.finalizeSources();
			concat.enableFuzzyMatching( 8 );

			// 3. Optionally set maxVoices. You won't be able to have 
			//    more than this many grains active at once.
			//    200 is the maximum.
			40 => concat.maxVoices;

			// 4. We match against the input to the concatenator. 
			//    Here let's use the mic.
			adc => Delay d => Gain intoConcat => concat;

			1::second => d.max;
			0.05::second => d.delay;

			// mic loudness!!!
			0.75 => d.gain;

			// (Also play the mic)
			d => reverbL;
			d => reverbR;

			global int matchedBuffer;
			global float inputLoudness;

			// check for silence
			global Event enableCheckForSilence;
			
			fun void CheckForSilence()
			{{
				enableCheckForSilence => now;
				now => time lastLoudSound;
				0.1 => float loudnessCutoff;
				7::second => dur timeToWaitBetweenInterjections;
				2::second => dur timeToWaitToInterJect;

				while( true )
				{{
					// measure / record
					if( concat.inputLoudness() > loudnessCutoff )
					{{
						now => lastLoudSound;
					}}

					// be patient
					if( now - lastLoudSound > timeToWaitToInterJect )
					{{
						spork ~ RespondToSilence();
						timeToWaitBetweenInterjections => now;
					}}

					// ok but check again real soon, y'hear?
					10::ms => now;
				}}
			}}
			spork ~ CheckForSilence();

			fun void RespondToSilence()
			{{
				// Play a random one of the sources
				SndBuf buf => intoConcat;
				Math.random2( 0, sources.size() - 1 ) => int which;
				me.dir() + ""objectSounds/"" + sources[ which ] => buf.read;
				// <<< ""imitating"", sources[which] >>>;
				buf.length() => now;

				buf =< intoConcat;
			}}


			global Event intensify;
			500::ms => dur grainLength;
			200::ms => dur grainTime;
			Gain echoesL => reverbL;
			Gain echoesR => reverbR;	

			fun void Intensify()
			{{
				20 => int N_ECHOES;
				Pan2 pans[N_ECHOES];
				Delay delays[N_ECHOES];
				0 => echoesL.gain => echoesR.gain; 

				for( int i; i < N_ECHOES; i++ )
				{{
					concat => delays[i] => pans[i];
					pans[i].left => echoesL;
					pans[i].right => echoesR;

					// select params randomly
					Math.random2f( 0.3, 4 )::second => delays[i].max => delays[i].delay;
					Math.random2f( 0.1, 0.4 ) => delays[i].gain;
					Math.random2f( -0.9, 0.9 ) => pans[i].pan;
				}}

				// ALAS: computer not powerful enough :(
				/*
				10 => int N_SAWS;
				Supersaw saws[N_SAWS];
				LPF sawsL => reverbL;
				LPF sawsR => reverbR;
				100 => sawsL.freq => sawsR.freq;

				for( int i; i < N_SAWS; i++ )
				{{
					saws[i] => sawsL;
					saws[i] => sawsR;

					// select params randomly
					// how much freq waves up and down as a % of current freq
					Math.random2f( 0.8, 3 ) / 300 => saws[i].pitchLFODepthMultiplier; 
					// how quickly the freq lfo moves
					Math.random2f( 0.5, 3.67 ) => saws[i].pitchLFORate;
					// higher == wider pitch spread
					Math.random2f( 0.1, 0.5 ) => saws[i].timbreLFO; 
					// basic loudness
					0.3 / N_SAWS => saws[i].gain;
					// amp LFO
					Math.random2f( 0.8, 3 ) => saws[i].ampModLFO;

					// pitch!
					Math.random2( 30, 55 ) => Std.mtof => saws[i].freq;
				}}
				*/


				intensify => now;

				1 => float goalGain;
				0 => float currentGain;

				0.2 => float goalReverb;
				0 => float currentReverb;

				/*
				2000 => float goalLPFFreq;
				sawsL.freq() => float currentLPFFreq;
				*/

				// 500, 200 --> 150, 50
				grainLength => dur currentGrainLength;
				grainTime => dur currentGrainTime;
				150::ms => dur goalGrainLength;
				50::ms => dur goalGrainTime;

				0.0015 => float allSlew;

				while( true )
				{{
					// gain: echoes, saws
					allSlew * ( goalGain - currentGain ) +=> currentGain;
					currentGain => echoesL.gain => echoesR.gain;
					// currentGain => sawsL.gain => sawsR.gain;

					// reverb
					allSlew * ( goalReverb - currentReverb ) +=> currentReverb;
					currentReverb => reverbL.mix => reverbR.mix;

					// grains
					allSlew * ( goalGrainLength - currentGrainLength ) +=> currentGrainLength;
					allSlew * ( goalGrainTime - currentGrainTime ) +=> currentGrainTime;
					currentGrainLength => grainLength;
					currentGrainTime => grainTime;

					// lpf cutoff
					/*
					allSlew * ( goalLPFFreq - currentLPFFreq ) +=> currentLPFFreq;
					currentLPFFreq => sawsL.freq => sawsR.freq;
					*/

					10::ms => now;
				}}
			}}
			spork ~ Intensify() @=> Shred intensifyShred;

			global Event deintensify;

			fun void Deintensify()
			{{
				deintensify => now;
				intensifyShred.exit();

				0 => float goalGain;
				echoesL.gain() => float currentGain;

				// 500, 200 <-- 150, 50
				grainLength => dur currentGrainLength;
				grainTime => dur currentGrainTime;
				500::ms => dur goalGrainLength;
				200::ms => dur goalGrainTime;

				0.015 => float allSlew;

				while( true )
				{{
					// echoes
					allSlew * ( goalGain - currentGain ) +=> currentGain;
					currentGain => echoesL.gain => echoesR.gain;

					// grains
					allSlew * ( goalGrainLength - currentGrainLength ) +=> currentGrainLength;
					allSlew * ( goalGrainTime - currentGrainTime ) +=> currentGrainTime;
					currentGrainLength => grainLength;
					currentGrainTime => grainTime;

					10::ms => now;
				}}
			}}
			spork ~ Deintensify();

			global Event fadeConcatSynth;
			global Event fadeVoice;

			fun void FadeUGen( UGen g, dur t )
			{{
				10::ms => dur delta;
				( t / delta ) $ int => int numSteps;
				g.gain() / numSteps => float gainDecrease;
				
				for( int i; i < numSteps; i++ )
				{{
					g.gain() - gainDecrease => g.gain;
					delta => now;
				}}
				0 => g.gain;
			}}

			fun void DoEnding()
			{{
				fadeConcatSynth => now;
				spork ~ FadeUGen( concat, 5::second );

				fadeVoice => now;
				spork ~ FadeUGen( reverbL, 5::second );
				spork ~ FadeUGen( reverbR, 5::second );

				10::second => now;
			}}
			spork ~ DoEnding();


			// 5. Trigger grains!
			while( true )
			{{
				// fuzzy matching to get a wider variety of sounds
				concat.triggerFuzzyGrain( grainLength, 0.5::grainLength, 0.5::grainLength );
				
				// store for Unity
				concat.matchedBuffer() => matchedBuffer;
				concat.inputLoudness() => inputLoudness;

				// wait until next grain
				grainTime => now;
			}}
		", theFiles ));

		myMatchedBuffer = gameObject.AddComponent<ChuckIntSyncer>();
		myMatchedBuffer.SyncInt( myChuck, "matchedBuffer" );
		myLoudness = gameObject.AddComponent<ChuckFloatSyncer>();
		myLoudness.SyncFloat( myChuck, "inputLoudness" );
	}

	void ShowObject( int which, float size )
	{
		// hide all
		for( int i = 0; i < myObjects.Length; i++ )
		{
			myObjects[i].gameObject.SetActive( false );
		}

		// set size
		myObjects[ which ].localScale = size * Vector3.one;

		// show
		myObjects[ which ].gameObject.SetActive( true );
	}
	
	// Update is called once per frame
	void Update()
	{
		// show match
		float scale = ( myLoudness.GetCurrentValue() + 1 ) / 2;
		int obj = myMatchedBuffer.GetCurrentValue();
		ShowObject( obj, scale );

		// spawn garbage if it's a new sound and has been > 0.15 sec
		// or if it's been same sound for 1 second
		if( shouldSpawnObjects && ( 
			obj != previousObject && Time.time - prevSpawnTime > 0.15
		    || Time.time - prevSpawnTime > 1 ) )
		{
			// spawn some garbage somewhere
			int which = Random.Range( 0, myPatches.Length );
			myPatches[ which ].GiveGarbage( myPrefabs[ obj ] );

			// remember spawn time
			prevSpawnTime = Time.time;
		}

		// remember
		previousObject = obj;

		// check
		if( Input.GetKeyDown("p") )
		{
			// turn on imitations
			// TODO disable?
			myChuck.BroadcastEvent( "enableCheckForSilence" );
			// turn on spawning
			shouldSpawnObjects = true;
		}

		if( Input.GetKeyDown("i") )
		{
			myChuck.BroadcastEvent( "intensify" );
		}

		if( Input.GetKeyDown("e") )
		{
			myChuck.BroadcastEvent( "deintensify" );
		}

		if( Input.GetKeyDown("q") )
		{
			myChuck.BroadcastEvent( "fadeConcatSynth" );
		}

		if( Input.GetKeyDown("w") )
		{
			myChuck.BroadcastEvent( "fadeVoice" );
		}
	}
}
