﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class ConcatSynthMilestone2 : MonoBehaviour 
{
	public Transform[] prefabs;

	public string[] soundFiles;

	public bool[] ignore;

	private ChuckSubInstance myChuck;
	private ChuckIntSyncer myMatchedBuffer;
	private ChuckFloatSyncer myLoudness;

	private Transform[] myObjects;

	// Use this for initialization
	void Start()
	{
		InitChuck();
		InitObjects();
	}

	void InitObjects()
	{
		int numToIgnore = 0;
		for( int i = 0; i < ignore.Length; i++ )
		{
			if( ignore[i] ) { numToIgnore++; }
		}

		myObjects = new Transform[ prefabs.Length - numToIgnore ];
		int which = 0;
		for( int i = 0; i < prefabs.Length; i++ )
		{
			if( !ignore[i] )
			{
				myObjects[which] = Instantiate( prefabs[i], transform );
				myObjects[which].position = Vector3.zero;
				myObjects[which].gameObject.SetActive( false );
				which++;
			}
		}
	}

	void InitChuck()
	{
		myChuck = GetComponent<ChuckSubInstance>();

		string theFiles = "";
		for( int i = 0; i < soundFiles.Length; i++ )
		{
			if( !ignore[i] )
			{
				theFiles += string.Format( @"concat.addSource( me.dir() + ""objectSounds/{0}"" );
			", soundFiles[i] );
			}
		}

		myChuck.RunCode( string.Format( @"
			// Let's concatenative-synthesis!
			ConcatSynth concat => dac;

			// 1. Optionally set number of MFCCs to be used in all analysis.
			//    13 is the default. 48 is the maximum. 
			//    We ignore the 0th MFCC, so you are actually using 
			//    one fewer than this.
			40 => concat.mfccCount;

			// 2. Add source sound files.
			{0}
			concat.finalizeSources();
			//concat.enableFuzzyMatching( 4 );

			// 3. Optionally set maxVoices. You won't be able to have 
			//    more than this many grains active at once.
			//    200 is the maximum.
			40 => concat.maxVoices;

			// 4. We match against the input to the concatenator. 
			//    Here let's use the mic, but delayed.
			// adc => Delay d => concat;
			//2::second => d.max => d.delay;

			// Just kidding let's just use the mic.
			adc => concat;

			global int matchedBuffer;
			global float inputLoudness;

			// 5. Trigger grains!
			while( true )
			{{
				// triggerGrain( grain length, ramp up time, ramp down time )
				// concat.triggerFuzzyGrain( 1000::ms, 500::ms, 500::ms );
				concat.triggerGrain( 500::ms, 500::ms, 500::ms );
				/*
				chout <= ""I matched buffer ""
					<= concat.matchedBuffer() 
					<= "" at sample "" 
					<= concat.matchedSample() 
					<= IO.newline();
				*/
				concat.matchedBuffer() => matchedBuffer;
				concat.inputLoudness() => inputLoudness;

				// 100::ms => now;
				200::ms => now;

			}}
		", theFiles ));

		myMatchedBuffer = gameObject.AddComponent<ChuckIntSyncer>();
		myMatchedBuffer.SyncInt( myChuck, "matchedBuffer" );
		myLoudness = gameObject.AddComponent<ChuckFloatSyncer>();
		myLoudness.SyncFloat( myChuck, "inputLoudness" );
	}

	void ShowObject( int which, float size )
	{
		// hide all
		for( int i = 0; i < myObjects.Length; i++ )
		{
			myObjects[i].gameObject.SetActive( false );
		}

		// set size
		myObjects[ which ].localScale = size * Vector3.one;

		// show
		myObjects[ which ].gameObject.SetActive( true );
	}
	
	// Update is called once per frame
	void Update()
	{
		// Debug.Log( "Matched " + soundFiles[ myMatchedBuffer.GetCurrentValue() ] );
		// Debug.Log( "Loudness " + myLoudness.GetCurrentValue() );
		float scale = ( myLoudness.GetCurrentValue() + 1 ) / 2;
		ShowObject( myMatchedBuffer.GetCurrentValue(), scale );
	}
}
