﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class ConcatSynthTest : MonoBehaviour {

	// Use this for initialization
	void Start () {
		GetComponent<ChuckSubInstance>().RunCode( @"
			// Let's concatenative-synthesis!
			ConcatSynth concat => dac;

			// 1. Optionally set number of MFCCs to be used in all analysis.
			//    13 is the default. 48 is the maximum. 
			//    We ignore the 0th MFCC, so you are actually using 
			//    one fewer than this.
			13 => concat.mfccCount;

			// 2. Add source sound files.
			concat.addSource( me.dir() + ""impact1.wav"" );
			concat.addSource( me.dir() + ""impact2.wav"" );
			concat.addSource( me.dir() + ""aah5.wav"" );
			concat.finalizeSources();
			concat.enableFuzzyMatching( 10 );

			// 3. Optionally set maxVoices. You won't be able to have 
			//    more than this many grains active at once.
			//    200 is the maximum.
			20 => concat.maxVoices;

			// 4. We match against the input to the concatenator. 
			//    Here let's use the mic.
			adc => concat;

			// 5. Trigger grains!
			while( true )
			{
				// triggerGrain( grain length, ramp up time, ramp down time )
				concat.triggerFuzzyGrain( 100::ms, 15::ms, 25::ms );
				chout <= ""I matched buffer ""
					<= concat.matchedBuffer() 
					<= "" at sample "" 
					<= concat.matchedSample() 
					<= IO.newline();
				30::ms => now;
			}

		
		" );
	}
	
	// Update is called once per frame
	void Update () {
		
	}
}
